import { getTestFiles, launch, isGlob, globFiles } from 'aria-mocha';
import { normalize, basename } from 'path';
import { existsSync, readFileSync } from 'fs';
import { createHash } from 'crypto';
import sade2 from 'sade';
import resolve from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import { rollup } from 'rollup';

const removeBacklash = (str) => {
  const paths = normalize(str).split("/");
  return paths.length > 1 ? paths.splice(1).join("/") : paths.shift();
};
const createUrl = ({port, hostname, path: path2}) => `http://${hostname}:${port}/${path2}`;
function normalizeOptions(options2) {
  const {
    script,
    headless = false,
    watch = false,
    hostname = "localhost",
    port = 3e3,
    dir = removeBacklash("test"),
    path: path2 = removeBacklash("tests"),
    html = removeBacklash("/node_modules/aria-vue/index.html")
  } = options2;
  return {script, watch, hostname, headless, port, dir, path: path2, html};
}

function testPlugin(options) {
  const {script, dir, path, html} = normalizeOptions(options);
  const Router = require("koa-router");
  const router = new Router();
  return ({app}) => {
    router.get("/test-files", async (ctx, next) => {
      ctx.body = await getTestFiles(`${dir}/**/*.spec.js`, true);
      await next();
    });
    router.get("/init", async (ctx, next) => {
      ctx.body = existsSync(script) ? [script] : [];
      await next();
    });
    app.use(async (ctx, next) => {
      if (ctx.path.includes(`/${path}`))
        return require("koa-send")(ctx, html);
      await next();
    });
    app.use(router.routes());
  };
}

const contentHash = (str) => createHash("md5").update(str.toString(), "utf8").digest("hex");
const getContentHash = (src) => contentHash(readFileSync(`.${src}`));
const getfileStore = async (files) => {
  const fileStore = new Map();
  const watchFiles = Array.isArray(files) ? files : [files];
  const substr = (str) => str.substr(1, str.length);
  const globs = await Promise.all(watchFiles.map(async (file) => {
    const files2 = isGlob(file) ? await globFiles(file, true) : [file];
    return files2.map((str) => substr(str));
  }));
  const flatFiles = globs.flat();
  await Promise.all(flatFiles.map((file) => {
    fileStore.set(file, getContentHash(file));
  }));
  const compare = (file) => {
    const current = getContentHash(file);
    return current !== fileStore.get(file) ? current : null;
  };
  const set = (file, content) => fileStore.set(file, content);
  return {compare, set};
};
function watchPlugin(options = {}) {
  const {port, path: path2, hostname, headless} = normalizeOptions(options);
  const {
    files = [
      "./src/**/*.vue",
      "./src/**/*.spec.js",
      "./test/**/*.spec.js"
    ]
  } = options;
  return async ({watcher, resolver}) => {
    const fileStore = await getfileStore(files);
    let fsWait = false;
    watcher.on("change", async (file) => {
      if (fsWait)
        return;
      fsWait = setTimeout(() => fsWait = false, 100);
      const pathFile = resolver.fileToRequest(file);
      const content = fileStore.compare(pathFile);
      if (content) {
        fileStore.set(pathFile, content);
        watcher.send({
          type: "full-reload",
          path: pathFile,
          timestamp: Date.now()
        });
        headless && await launch(createUrl({
          port,
          hostname,
          path: pathFile.includes(".spec.js") ? `${path2}?spec=${basename(pathFile)}` : path2
        }));
      }
    });
  };
}

function patchConfig() {
  const config = {
    ...existsSync("./node_modules/@babel/parser/index.js") ? {
      alias: {
        "@babel/parser": "@babel/parser/index"
      }
    } : {}
  };
  return config;
}
function createVueTestPlugin(options) {
  const opts = normalizeOptions(options);
  const config = {
    ...patchConfig(),
    configureServer: [
      testPlugin(opts),
      ...opts.watch ? [watchPlugin(opts)] : []
    ]
  };
  return config;
}

async function launchHeadless({hostname, port, path, watch}) {
  await launch(createUrl({hostname, port, path}));
  !watch && process.exit();
}
async function startServer(options2) {
  const opts = normalizeOptions(options2);
  const {createServer} = await import('vite');
  const vueTestPlugin = createVueTestPlugin(opts);
  const server = createServer(vueTestPlugin);
  const {port, headless, hostname, html} = opts;
  server.listen(port, hostname, async () => {
    headless ? await launchHeadless({...opts, path: html}) : console.log(`Go to ${createUrl(opts)}`);
  });
}

async function patchBabelParser() {
  const bundle = await rollup({
    input: "./node_modules/aria-vue/tools/babel-parser.js",
    plugins: [commonjs(), resolve()]
  });
  await bundle.write({
    file: "./node_modules/@babel/parser/index.js",
    format: "es",
    sourcemap: true
  });
}

async function run({version, name}) {
  const prog = sade2(name, true);
  prog.version(version).option("-p, --port", "port to use default(3000)", 3e3).option("-d --dir", "root directory of test files default (test folder)", "test").option("-w --watch", "enable watch", false).option("-H --headless", "run test(s) in headless", false).option("--script", "scripts or helper scripts to load before setup").option("--path", "virtual path for your html reporter").option("--html", "path of your index.html file").example("--headless --script ./test/plugin.js").example("--watch --path my-virtual-path --script ./test/plugin.js").example("--path test-ui --html ./test/index.html --script ./test/plugins.js").action(handler).parse(process.argv);
}
async function handler(options2) {
  if (!existsSync("./node_modules/@babel/parser/index.js")) {
    await patchBabelParser();
  }
  await startServer(options2);
}

export { createVueTestPlugin, handler, patchBabelParser, run, startServer };
